import glob
import os
import sys
import argparse
from argparse import RawTextHelpFormatter
from Tkinter import *
import Tkinter,tkFileDialog
import subprocess
from subprocess import check_output
from subprocess import call


#=============================================== ANTS DIRECTORY (manual input) ===========================================================

# type the location of your ANTS directory and change the variable ask_for_ants_dir to 0. If this variable is set to 1, the program will ask for the ANTS directory at the beginning

#ants_dir = '<type your ANTS directory>'
ask_for_ants_dir = 1

#=========================================================================================================================================


#======================================================= DOCUMENTATION ===================================================================

parser = argparse.ArgumentParser(
	description = "How to use the automated classification program:\n\n    1) Prepare the files to be analyzed. An anatomical image and the segmented lesion image are required for each subject. Importantly, both images must have the same name, and they have to be distinguished from each other by placing the suffix '_anatomic' (for the anatomical image) and '_segmented' (for the segmented lesion image), after the name and before the extension of the file (e.g., 'subject1_anatomic.nii' and 'subject1_segmented.nii'). All images must be located in a folder that has to be placed in the same directory where the supplementary folders are located.\n\n    2) First, you will be prompted to type a name for the resulting files that will be generated. The name by default is 'output'.\n\n    3) Second, you will have to specify the directory in which your ANTs software is located. If you are using the Python script instead of the executable, you can modify it in order to avoid having to select the ANTs directory every time you run the program. To do so, open the script with a text processor, replace the value of the variable 'ants_dir' (line 17) with the path of your ANTs directory, and uncomment the line; additionally, change the value of the variable 'ask_for_ants_dir' (line 18) to 0.\n\n\n      Note: in order to select a directory when you are prompted to, you need to search for it and double-click it. The name of the desired folder must appear, along with the rest of the path, in the textbox located in the bottom of the dialog window.\n\n\n    4) Finally, you will have to select the directory containing the required files (i.e., both the segmentation and the anatomic images of the subject/s).\n\n\n      Important: two new folders will be created in this directory; one of them (named 'generated_files') will contain all the files generated during the procedure, while the other (named 'results') will contain the files with your results. If you choose a directory that has been previously analyzed, you will be asked to confirm whether you want to overwrite the results or not. This may be a good option if you want to ADD new subjects to those that have been already analyzed without having to repeat the whole procedure, but note that you must keep the older files in this directory; otherwise, the previous results will be deleted.\n\n\n    5) (Optional) The degree up to which the resulting image is smoothed can be adjusted if you are using the Python script instead of the executable. You can do it by replacing the value of the variable 'sigma' (line 436), which is set by default to 0.85. Greater values of 'sigma' will result in a higher degree of smoothing.\n\n\n\n\nOutput:\n\n      The following 3 files will be created in the 'results' folder that will be generated in the directory where your anatomical and segmented images are located:\n\n\n    1) Excel file: the Excel spreadsheet contains 3 sheets. In the first sheet ('General information'), absolute and relative lesion volumes are shown for the full brain and for each hemisphere. The second sheet ('Lesion location') shows how each lesion extends over the arterial territories, i.e., the percentage of lesion located at each arterial territory. The third sheet ('Territory affectation') shows how each arterial territory is affected by the lesion, i.e., the percentage of territory involved in each lesion.\n\n    2) Bar graph: the graph depicts a bar for each lesion analyzed. The bars are segmented according to the percentage of lesion located at each arterial territory. The information depicted in the graph is equivalent to that shown in the sheet 'Lesion location' of the Excel spreadsheet.\n\n    3) Lesion distribution image: this three-dimensional image shows all the segmented lesions overlapped and warped to the ATNB map. Since the volume of each lesion is binarized, this image can be interpreted as a 'heat map', in which greater values indicate a higher concentration of lesions. This image can then be overlaid to the ATNB map to have a graphical representation of how the lesions are distributed over the territories.\n", formatter_class=RawTextHelpFormatter)

args = parser.parse_args()

#=========================================================================================================================================


# open dialog to type a name for the graph, the spreadsheet, and the image with overlapped lesions (only if there is more than 1)

root = Tk()
label = Label(root, text="Choose a name for the spreadsheet", width=40, height=2)
entry = Entry(root,bd=5)

def choose_name():

	global savename
	savename = entry.get()
	print('\nThe spreadsheet will be saved as ' + savename + '.xls\n')
	root.withdraw()
	root.destroy()


submit = Button(root, text="OK", command=choose_name)


label.pack()
entry.pack()
submit.pack(side=BOTTOM)
entry.insert(0,"output")

root.mainloop()


if ask_for_ants_dir == 1:

	# open dialog to choose the directory where ANTS is located if it has not been defined previously

	root2 = Tkinter.Tk()
	ants_dir = tkFileDialog.askdirectory(initialdir=".",parent=root2,title='Choose your ANTS directory')
	root2.withdraw()

else:

	None


# open dialog to select file(s)

root3 = Tkinter.Tk()
files_dir = tkFileDialog.askdirectory(initialdir=".",parent=root3,title='Choose directory containing files to analyze')
root3.withdraw()

anatomic_files = []
segmented_files = []

os.chdir(files_dir)

for file in glob.glob("*_anatomic.nii*"):

	anatomic_files.append(file)

for file2 in glob.glob("*_segmented.nii*"):

	segmented_files.append(file2)


# check number of files selected and ask for confirmation

numb_anatomic_files = len(anatomic_files)
numb_segmented_files = len(segmented_files)


if numb_anatomic_files != numb_segmented_files:

	sys.exit("Error: different number of anatomic and segmented files")

else:

	None
	

# remove path and extension to get the actual name of the files

name_files = []
anatomic_exts = []

i = 0

while i < numb_anatomic_files:

	no_path_name = anatomic_files[i].split("/")[-1]
	no_ext_name = no_path_name.split(".")[0]
	ext_anatomic_name = no_path_name.split(".")[-1]

	if ext_anatomic_name == "gz":

		ext_anatomic_name = "nii.gz"

	else:

		None

	anatomic_exts.append(ext_anatomic_name)
	actual_name = no_ext_name.rsplit('_', 1)[0]
	name_files.append(actual_name)

	i += 1

i = 0

segmented_exts = []

while i < numb_segmented_files:

	no_path_name = segmented_files[i].split("/")[-1]
	ext_segmented_name = no_path_name.split(".")[-1]

	if ext_segmented_name == "gz":

		ext_segmented_name = "nii.gz"

	else:

		None

	segmented_exts.append(ext_segmented_name)

	i += 1


if numb_anatomic_files == 1:

		print(str(numb_anatomic_files) + ' file has been selected.\n')

else:

	if numb_anatomic_files == 0:

		print('No segmented lesions were found\n')
		os._exit(1)

	else:

		print(str(numb_anatomic_files) + ' files have been selected.\n')


raw_input("Press Enter to continue or Ctrl+Z to cancel.\n")


if not os.path.exists(files_dir + "/generated_files"):

	os.makedirs("generated_files")
	os.makedirs("results")

else:

	print('\n\nAt least some files from this directory appear to have been analyzed before. Do you wish to continue?\n')

	raw_input("Press Enter to continue or Ctrl+Z to cancel.\n")


print '\nStarting...\n'


os.chdir("..")


brain_vol = 394692 # total brain volume of the map (mm^3)
hemisphere_vol = 197346 # hemisphere volume (mm^3)

# list and number of arterial territories

terr_list = ['L_ACA.nii.gz','L_ACA_per.nii.gz','L_AICA.nii.gz','L_CA.nii.gz','L_MCA_ant.nii.gz','L_MCA_per.nii.gz','L_MCA_post.nii.gz','L_PCA.nii.gz','L_PCA_per.nii.gz','L_PICA.nii.gz','L_SCA.nii.gz','R_ACA.nii.gz','R_ACA_per.nii.gz','R_AICA.nii.gz','R_CA.nii.gz','R_MCA_ant.nii.gz','R_MCA_per.nii.gz','R_MCA_post.nii.gz','R_PCA.nii.gz','R_PCA_per.nii.gz','R_PICA.nii.gz','R_SCA.nii.gz'] # list of arterial territories
numb_terr = len(terr_list) # number of territories


# calculate the volume for each arterial territory

terr_volumes = []


i = 0

while i < numb_terr:

	output_terr_volume = check_output(["fsl5.0-fslstats arterial_territories/{} -V".format(terr_list[i])], shell=True)
	terr_voxels, terr_volume = output_terr_volume.split()
	terr_volumes.append(int(float(terr_volume)))

	i += 1


# warp selected files to the arterial map brain

warning = []

i = 0

while i < numb_anatomic_files:

	print('\nProcessing subject {}'.format(name_files[i]) + ' (' + str(i+1) + ' of ' + str(numb_anatomic_files) + ')\n')

	if os.path.isfile("{}/generated_files/{}_warped_segmentation.nii.gz".format(files_dir,name_files[i])) == True:

		print('        Warped segmentation image already exists. This step will be skipped\n')

		i += 1

	else:

		print('        Warping segmentation image to the arterial map brain\n')

		call(["{}/ImageMath 3 {}/generated_files/{}_neg.nii.gz Neg {}/{}_segmented.{}".format(ants_dir,files_dir,name_files[i],files_dir,name_files[i],segmented_exts[i])], shell=True)

		call(["{}/ImageMath 3 {}/generated_files/{}_smoothed_neg.nii.gz G {}/generated_files/{}_neg.nii.gz 1".format(ants_dir,files_dir,name_files[i],files_dir,name_files[i])], shell=True)

		call(["fsl5.0-fslmaths {}/{}_anatomic -mul {}/generated_files/{}_smoothed_neg {}/generated_files/{}_masked_anatomic".format(files_dir,name_files[i],files_dir,name_files[i],files_dir,name_files[i])], shell=True)

		call(["fsl5.0-flirt -in {}/generated_files/{}_masked_anatomic.nii.gz -ref ref_brain/JER_T1.nii.gz -out {}/generated_files/{}_affine_masked_anatomic -omat {}/generated_files/{}_rigid_anatomic_to_T1.mat -dof 6".format(files_dir,name_files[i],files_dir,name_files[i],files_dir,name_files[i])], shell=True)

		call(["fsl5.0-flirt -in {}/{}_segmented.{} -ref ref_brain/JER_T1.nii.gz -out {}/generated_files/{}_affine -init {}/generated_files/{}_rigid_anatomic_to_T1.mat -applyxfm".format(files_dir,name_files[i],segmented_exts[i],files_dir,name_files[i],files_dir,name_files[i])], shell=True)

		call(["ANTS 3 -m CC[ref_brain/JER_T1.nii.gz,{}/generated_files/{}_affine_masked_anatomic.nii.gz,1,2] -i 10x50x50x20 -o {}/generated_files/{}_anatomic_to_2w_T1_ -t SyN[0.25] -r Gauss[3.0]".format(files_dir,name_files[i],files_dir,name_files[i])], shell=True)

		call(["WarpImageMultiTransform 3 {}/generated_files/{}_affine_masked_anatomic.nii.gz {}/generated_files/{}_warped_anatomic.nii.gz -R ref_brain/JER_T1.nii.gz {}/generated_files/{}_anatomic_to_2w_T1_Warp.nii.gz {}/generated_files/{}_anatomic_to_2w_T1_Affine.txt".format(files_dir,name_files[i],files_dir,name_files[i],files_dir,name_files[i],files_dir,name_files[i])], shell=True)

		call(["WarpImageMultiTransform 3 {}/generated_files/{}_affine.nii.gz {}/generated_files/{}_pre_warped_segmentation.nii.gz -R ref_brain/JER_T1.nii.gz {}/generated_files/{}_anatomic_to_2w_T1_Warp.nii.gz {}/generated_files/{}_anatomic_to_2w_T1_Affine.txt".format(files_dir,name_files[i],files_dir,name_files[i],files_dir,name_files[i],files_dir,name_files[i])], shell=True)

		call(["fsl5.0-fslmaths {}/generated_files/{}_pre_warped_segmentation.nii.gz -mul 0.5 {}/generated_files/{}_warped_segmentation.nii.gz".format(files_dir,name_files[i],files_dir,name_files[i])], shell=True)


		# check minimum and maximum intensity in the image

		output_intensity = check_output(["fsl5.0-fslstats {}/generated_files/{}_warped_segmentation.nii.gz -R".format(files_dir,name_files[i])], shell=True)
		min_int, max_int = output_intensity.split()
		min_int_split = min_int.split('.')
		max_int_split = max_int.split('.')

		if int(min_int_split[0]) != 0:

			print('                WARNING!: minimum intensity is not 0. Check this file as it may alter the results')
			warning.append('Subject ' + name_files[i] + ': WARNING! minimum intensity is not 0. Check this file as it may alter the results')

		elif int(max_int_split[0]) != 1:

			print('                WARNING!: maximum intensity is not 1. Check this file as it may alter the results')
			warning.append('Subject ' + name_files[i] + ': WARNING! maximum intensity is not 1. Check this file as it may alter the results')

		else:

			print('        Generated warped image is OK\n\n')

		i += 1


# create a text file with all the errors found (if any)

if len(warning) > 0:

	i = 0

	while i < len(warning):

		warning_file = open('warning.txt', 'a')
		warning_file.write(warning[i] + "\n")

		i += 1

else:

	None


# define variables for the following steps

total_vol = [] # total lesion volumes will be added here
total_rel_vol = [] # relative lesion volumes will be added here (considering total brain volume of the map)
lh_vol = [] # left hemisphere lesion volumes will be added here
lh_rel_vol = [] # relative left hemisphere lesion volumes will be added here (considering total left hemisphere volume of the map)
rh_vol = [] # right hemisphere lesion volumes will be added here
rh_rel_vol = [] # relative right hemisphere lesion volumes will be added here (considering total right hemisphere volume of the map)
lesion_terr_vol = [[0 for x in range(numb_anatomic_files)] for x in range(numb_terr)] # total volume of affectation of each arterial territory will be added here 
lesion_terr_rel_vol = [[0 for x in range(numb_anatomic_files)] for x in range(numb_terr)] # relative % of affectation of each arterial territory will be added here (considering each arterial territory volume)
lesion_locat_vol = [[0 for x in range(numb_anatomic_files)] for x in range(numb_terr)] # % of lesion located in each arterial territory will be added here


# calculate volumes for the files selected after warping

i = 0

while i < numb_anatomic_files:

	print('Calculating volumes from subject {}'.format(name_files[i]) + ' (' + str(i+1) + ' of ' + str(numb_anatomic_files) + ')\n')


	# calculate volumes for the left hemisphere

	output_lh = check_output(["fsl5.0-fslstats {}/generated_files/{}_warped_segmentation.nii.gz -k masks/lh_mask.nii.gz -V".format(files_dir,name_files[i])], shell=True)
	lh_voxels, lh_volume = output_lh.split()
	lh_vol.append(int(float(lh_volume)))
	lh_rel_vol.append((1. * int(float(lh_volume))/hemisphere_vol) * 100)

	if lh_rel_vol[i] == 0:

		lh_rel_vol[i] = 0

	else:

		lh_rel_vol[i] = "%.2f" % (lh_rel_vol[i])


	# calculate volumes for the right hemisphere

	output_rh = check_output(["fsl5.0-fslstats {}/generated_files/{}_warped_segmentation.nii.gz -k masks/rh_mask.nii.gz -V".format(files_dir,name_files[i])], shell=True)
	rh_voxels, rh_volume = output_rh.split()
	rh_vol.append(int(float(rh_volume)))
	rh_rel_vol.append((1. * int(float(rh_volume))/hemisphere_vol) * 100)

	if rh_rel_vol[i] == 0:

		rh_rel_vol[i] = 0

	else:

		rh_rel_vol[i] = "%.2f" % (rh_rel_vol[i])


	# calculate volumes for the total brain

	total_vol.append(lh_vol[i]+rh_vol[i])
	total_rel_vol.append((1. * int(float(total_vol[i]))/brain_vol) * 100)

	if total_rel_vol[i] == 0:

		total_rel_vol[i] = 0

	else:

		total_rel_vol[i] = "%.2f" % (total_rel_vol[i])


	# new loop to calculate the % of each arterial territory affected by the lesion

	i2 = 0

	while i2 < numb_terr:

		output_lesion_terr = check_output(["fsl5.0-fslstats {}/generated_files/{}_warped_segmentation.nii.gz -k arterial_territories/{} -V".format(files_dir,name_files[i],terr_list[i2])], shell=True)
		lesion_terr_voxels, lesion_terr_volume = output_lesion_terr.split()
		lesion_terr_vol[i2][i] = int(float(lesion_terr_volume))
		lesion_terr_rel_vol[i2][i] = (1. * lesion_terr_vol[i2][i] / terr_volumes[i2]) * 100

		if lesion_terr_rel_vol[i2][i] == 0:

			lesion_terr_rel_vol[i2][i] = 0

		else:

			lesion_terr_rel_vol[i2][i] = "%.2f" % (lesion_terr_rel_vol[i2][i])
	
		i2 += 1


	# new loop to calculate the % of lesion located at each arterial territory

	i2 = 0
	while i2 < numb_terr:
		lesion_locat_vol[i2][i] = (1. * lesion_terr_vol[i2][i] / total_vol[i]) * 100

		if lesion_locat_vol[i2][i] == 0:

			lesion_locat_vol[i2][i] = 0

		else:

			lesion_locat_vol[i2][i] = "%.2f" % (lesion_locat_vol[i2][i])
	
		i2 += 1
	
	i += 1


# generate image with the lesions overlapped (only if there is more than 1)


if numb_anatomic_files > 1:

	print('\nOverlapping all lesions in a single image\n')

	# create zeroed image

	call(["fsl5.0-fslmaths {}/generated_files/{}_warped_segmentation.nii.gz -mul 0 {}/generated_files/{}.nii.gz".format(files_dir,name_files[0],files_dir,savename)], shell=True)


	# loop to overlap each lesion

	i = 0
	while i < numb_anatomic_files:

		call(["fsl5.0-fslmaths {}/generated_files/{}.nii.gz -add {}/generated_files/{}_warped_segmentation.nii.gz {}/generated_files/{}.nii.gz".format(files_dir,savename,files_dir,name_files[i],files_dir,savename)], shell=True)
		
		i += 1


	# smooth completed image
	
	sigma = 0.85 # this value can be decreased or increased to get a lower or higher degree of smoothing, respectively

	call(["{}/ImageMath 3 {}/generated_files/{}_smoothed.nii.gz G {}/generated_files/{}.nii.gz {}".format(ants_dir,files_dir,savename,files_dir,savename,sigma)], shell=True)


	# mask smoothed completed image

	call(["fsl5.0-fslmaths {}/generated_files/{}_smoothed.nii.gz -mul masks/overlapped_lesions_mask.nii.gz -mul 0.5 {}/results/{}_smoothed_masked.nii.gz".format(files_dir,savename,files_dir,savename)], shell=True)

else:

	None


print('\nGenerating graph and spreadsheet\n')

import plotly
import plotly.graph_objs as go
from plotly.graph_objs import Scatter, Layout
from xlwt import *

# generate graphs

trace1 = go.Bar(y=name_files,x=lesion_locat_vol[0],name='Left ACA',hoverinfo='name',orientation='h',marker=dict(color='rgba(254, 228, 0, 0.6)',line=dict(color='rgba(255, 0, 0, 1.0)',width=3)))
trace2 = go.Bar(y=name_files,x=lesion_locat_vol[11],name='Right ACA',hoverinfo='name',orientation='h',marker=dict(color='rgba(254, 228, 0, 0.6)',line=dict(color='rgba(0, 0, 255, 1.0)',width=3)))
trace3 = go.Bar(y=name_files,x=lesion_locat_vol[1],name='Left perf. ACA',hoverinfo='name',orientation='h',marker=dict(color='rgba(255, 126, 0, 0.6)',line=dict(color='rgba(255, 0, 0, 1.0)',width=3)))
trace4 = go.Bar(y=name_files,x=lesion_locat_vol[12],name='Right perf. ACA',hoverinfo='name',orientation='h',marker=dict(color='rgba(255, 126, 0, 0.6)',line=dict(color='rgba(0, 0, 255, 1.0)',width=3)))
trace5 = go.Bar(y=name_files,x=lesion_locat_vol[4],name='Left ant. MCA',hoverinfo='name',orientation='h',marker=dict(color='rgba(0, 221, 0, 0.6)',line=dict(color='rgba(255, 0, 0, 1.0)',width=3)))
trace6 = go.Bar(y=name_files,x=lesion_locat_vol[15],name='Right ant. MCA',hoverinfo='name',orientation='h',marker=dict(color='rgba(0, 221, 0, 0.6)',line=dict(color='rgba(0, 0, 255, 1.0)',width=3)))
trace7 = go.Bar(y=name_files,x=lesion_locat_vol[6],name='Left post. MCA',hoverinfo='name',orientation='h',marker=dict(color='rgba(254, 128, 254, 0.6)',line=dict(color='rgba(255, 0, 0, 1.0)',width=3)))
trace8 = go.Bar(y=name_files,x=lesion_locat_vol[17],name='Right post. MCA',hoverinfo='name',orientation='h',marker=dict(color='rgba(254, 128, 254, 0.6)',line=dict(color='rgba(0, 0, 255, 1.0)',width=3)))
trace9 = go.Bar(y=name_files,x=lesion_locat_vol[5],name='Left perf. MCA',hoverinfo='name',orientation='h',marker=dict(color='rgba(0, 144, 87, 0.6)',line=dict(color='rgba(255, 0, 0, 1.0)',width=3)))
trace10 = go.Bar(y=name_files,x=lesion_locat_vol[16],name='Right perf. MCA',hoverinfo='name',orientation='h',marker=dict(color='rgba(0, 144, 87, 0.6)',line=dict(color='rgba(0, 0, 255, 1.0)',width=3)))
trace11 = go.Bar(y=name_files,x=lesion_locat_vol[3],name='Left CA',hoverinfo='name',orientation='h',marker=dict(color='rgba(254, 0, 0, 0.6)',line=dict(color='rgba(255, 0, 0, 1.0)',width=3)))
trace12 = go.Bar(y=name_files,x=lesion_locat_vol[14],name='Right CA',hoverinfo='name',orientation='h',marker=dict(color='rgba(254, 0, 0, 0.6)',line=dict(color='rgba(0, 0, 255, 1.0)',width=3)))
trace13 = go.Bar(y=name_files,x=lesion_locat_vol[7],name='Left PCA',hoverinfo='name',orientation='h',marker=dict(color='rgba(82, 237, 254, 0.6)',line=dict(color='rgba(255, 0, 0, 1.0)',width=3)))
trace14 = go.Bar(y=name_files,x=lesion_locat_vol[18],name='Right PCA',hoverinfo='name',orientation='h',marker=dict(color='rgba(82, 237, 254, 0.6)',line=dict(color='rgba(0, 0, 255, 1.0)',width=3)))
trace15 = go.Bar(y=name_files,x=lesion_locat_vol[8],name='Left perf. PCA',hoverinfo='name',orientation='h',marker=dict(color='rgba(24, 101, 254, 0.6)',line=dict(color='rgba(255, 0, 0, 1.0)',width=3)))
trace16 = go.Bar(y=name_files,x=lesion_locat_vol[19],name='Right perf. PCA',hoverinfo='name',orientation='h',marker=dict(color='rgba(24, 101, 254, 0.6)',line=dict(color='rgba(0, 0, 255, 1.0)',width=3)))
trace17 = go.Bar(y=name_files,x=lesion_locat_vol[10],name='Left SCA',hoverinfo='name',orientation='h',marker=dict(color='rgba(164, 117, 213, 0.6)',line=dict(color='rgba(255, 0, 0, 1.0)',width=3)))
trace18 = go.Bar(y=name_files,x=lesion_locat_vol[21],name='Right SCA',hoverinfo='name',orientation='h',marker=dict(color='rgba(164, 117, 213, 0.6)',line=dict(color='rgba(0, 0, 255, 1.0)',width=3)))
trace19 = go.Bar(y=name_files,x=lesion_locat_vol[2],name='Left AICA',hoverinfo='name',orientation='h',marker=dict(color='rgba(219, 19, 59, 0.6)',line=dict(color='rgba(255, 0, 0, 1.0)',width=3)))
trace20 = go.Bar(y=name_files,x=lesion_locat_vol[13],name='Right AICA',hoverinfo='name',orientation='h',marker=dict(color='rgba(219, 19, 59, 0.6)',line=dict(color='rgba(0, 0, 255, 1.0)',width=3)))
trace21 = go.Bar(y=name_files,x=lesion_locat_vol[9],name='Left PICA',hoverinfo='name',orientation='h',marker=dict(color='rgba(232, 149, 121, 0.6)',line=dict(color='rgba(255, 0, 0, 1.0)',width=3)))
trace22 = go.Bar(y=name_files,x=lesion_locat_vol[20],name='Right PICA',hoverinfo='name',orientation='h',marker=dict(color='rgba(232, 149, 121, 0.6)',line=dict(color='rgba(0, 0, 255, 1.0)',width=3)))


data = [trace1,trace2,trace3,trace4,trace5,trace6,trace7,trace8,trace9,trace10,trace11,trace12,trace13,trace14,trace15,trace16,trace17,trace18,trace19,trace20,trace21,trace22]
layout = go.Layout(barmode='stack',hovermode='closest',title='Lesion location: % of lesion located at each arterial territory')
fig = go.Figure(data=data,layout=layout)
plotly.offline.plot(fig,filename=files_dir + '/results/' + savename + '.html')


# write info on a spreadsheet

wb = Workbook()


# spreadsheet styles

fnt1 = Font();fnt1.bold=True;fnt3=Font();fnt3.bold=True
brd1 = Borders();brd1.left=2;brd1.right=2;brd1.top=2;brd1.bottom=2
brd2 = Borders();brd2.left=0;brd2.right=2;brd2.top=0;brd2.bottom=2
brd3 = Borders();brd3.left=0;brd3.right=2;brd3.top=0;brd3.bottom=2
brd4 = Borders();brd4.left=0;brd4.right=2;brd4.top=2;brd4.bottom=2
brd5 = Borders();brd5.left=0;brd5.right=0;brd5.top=0;brd5.bottom=2
brd6 = Borders();brd6.left=2;brd6.right=2;brd6.top=2;brd6.bottom=2
brd7 = Borders();brd7.left=0;brd7.right=1;brd7.top=0
brd9 = Borders();brd9.left=0;brd9.right=2;brd9.top=2;brd9.bottom=2
brd10 = Borders();brd10.left=0;brd10.right=0;brd10.top=2;brd10.bottom=0
brd11 = Borders();brd11.left=0;brd11.right=0;brd11.top=2;brd11.bottom=2
pat1 = Pattern();pat1.pattern=Pattern.SOLID_PATTERN;pat1.pattern_fore_colour=0x2C
pat3 = Pattern();pat3.pattern=Pattern.SOLID_PATTERN;pat3.pattern_fore_colour=0x1D
pat3_2 = Pattern();pat3_2.pattern=Pattern.SOLID_PATTERN;pat3_2.pattern_fore_colour=0x28
pat4 = Pattern();pat4.pattern=Pattern.SOLID_PATTERN;pat4.pattern_fore_colour=0x2B
al1 = Alignment();al1.horz=Alignment.HORZ_CENTER;al1.vert=Alignment.VERT_CENTER
al2 = Alignment();al2.horz=Alignment.HORZ_RIGHT;al2.vert=Alignment.VERT_CENTER
style1 = XFStyle();style1.font=fnt1;style1.borders=brd1;style1.pattern=pat1;style1.alignment=al1
style2 = XFStyle();style2.borders=brd2;style2.alignment=al1
style3 = XFStyle();style3.font=fnt3;style3.borders=brd3;style3.pattern=pat3;style3.alignment=al1
style3_2 = XFStyle();style3_2.font=fnt3;style3_2.borders=brd3;style3_2.pattern=pat3_2;style3_2.alignment=al1
style3_head = XFStyle();style3_head.font=fnt3;style3_head.borders=brd4;style3_head.pattern=pat3;style3_head.alignment=al1
style3_2_head = XFStyle();style3_2_head.font=fnt3;style3_2_head.borders=brd4;style3_2_head.pattern=pat3_2;style3_2_head.alignment=al1
style4 = XFStyle();style4.font=fnt3;style4.borders=brd4;style4.pattern=pat4;style4.alignment=al1
style5 = XFStyle();style5.font=fnt3;style5.borders=brd5;style5.pattern=pat3;style5.alignment=al1
style5_2 = XFStyle();style5_2.font=fnt3;style5_2.borders=brd5;style5_2.pattern=pat3_2;style5_2.alignment=al1
style5_head = XFStyle();style5_head.font=fnt3;style5_head.borders=brd11;style5_head.pattern=pat3;style5_head.alignment=al1
style5_2_head = XFStyle();style5_2_head.font=fnt3;style5_2_head.borders=brd11;style5_2_head.pattern=pat3_2;style5_2_head.alignment=al1
style6 = XFStyle();style6.font=fnt3;style6.borders=brd6;style6.pattern=pat4;style6.alignment=al1
style7 = XFStyle();style7.borders=brd7;style7.alignment=al2
style8 = XFStyle();style8.alignment=al2
style9 = XFStyle();style9.font=fnt3;style9.borders=brd9;style9.pattern=pat3;style9.alignment=al1
style10 = XFStyle();style10.borders=brd10


# write sheet 1

ws1 = wb.add_sheet("General information",cell_overwrite_ok=True)

ws1.write_merge(0, 2, 0, 10, 'General information', style1)
ws1.write_merge(3, 4, 0, 10, 'Information regarding absolute and relative volumes of the lesion in the brain and left and right hemispheres', style2)
ws1.write_merge(6, 6, 1, 3, 'Absolute lesion volumes (mm^3)', style6)
ws1.write_merge(6, 6, 4, 6, 'Relative lesion volumes (%)', style4)
ws1.write(7, 0, 'Subject', style9)
ws1.write(7, 1, 'Full brain', style5)
ws1.write(7, 2, 'Left hem.', style5)
ws1.write(7, 3, 'Right hem.', style3)
ws1.write(7, 4, 'Full brain', style5)
ws1.write(7, 5, 'Left hem.', style5)
ws1.write(7, 6, 'Right hem.', style3)
ws1.write_merge(7, 7, 7, 10, 'File analyzed', style9)


i = 0
j = 0
e = 0

while i < numb_anatomic_files:

	if j == 20:

		ws1.write(i+8+e, 0, 'Subject', style9)
		ws1.write(i+8+e, 1, 'Full brain', style5_head)
		ws1.write(i+8+e, 2, 'Left hem.', style5_head)
		ws1.write(i+8+e, 3, 'Right hem.', style3_head)
		ws1.write(i+8+e, 4, 'Full brain', style5_head)
		ws1.write(i+8+e, 5, 'Left hem.', style5_head)
		ws1.write(i+8+e, 6, 'Right hem.', style3_head)
		ws1.write_merge(i+8+e, i+8+e, 7, 10, 'File analyzed', style9)

		j = 0
		e += 1

	else:

		None

	ws1.write(i+8+e, 0, name_files[i], style7)
	ws1.write(i+8+e, 1, total_vol[i], style8)
	ws1.write(i+8+e, 2, lh_vol[i], style8)
	ws1.write(i+8+e, 3, rh_vol[i], style7)
	ws1.write(i+8+e, 4, total_rel_vol[i], style8)
	ws1.write(i+8+e, 5, lh_rel_vol[i], style8)
	ws1.write(i+8+e, 6, rh_rel_vol[i], style7)
	ws1.write_merge(i+8+e, i+8+e, 7, 10, segmented_files[i], style7)

	i += 1
	j += 1


ws1.write(i+8+e, 0, '', style10)
ws1.write(i+8+e, 1, '', style10)
ws1.write(i+8+e, 2, '', style10)
ws1.write(i+8+e, 3, '', style10)
ws1.write(i+8+e, 4, '', style10)
ws1.write(i+8+e, 5, '', style10)
ws1.write(i+8+e, 6, '', style10)
ws1.write(i+8+e, 7, '', style10)
ws1.write(i+8+e, 8, '', style10)
ws1.write(i+8+e, 9, '', style10)
ws1.write(i+8+e, 10, '', style10)


# write sheet 2

ws2 = wb.add_sheet("Lesion location",cell_overwrite_ok=True)

ws2.write_merge(0, 2, 0, 10, 'Lesion location', style1)
ws2.write_merge(3, 4, 0, 10, 'Percentage of lesion located at each arterial territory', style2)
ws2.write_merge(6, 6, 1, 11, 'Left hemisphere', style6)
ws2.write_merge(6, 6, 12, 22, 'Right hemisphere', style4)
ws2.write(7, 0, 'Subject', style9)
ws2.write(7, 1, 'ACA', style5)
ws2.write(7, 2, 'Perf. ACA', style5)
ws2.write(7, 3, 'Ant. MCA', style5)
ws2.write(7, 4, 'Post. MCA', style5)
ws2.write(7, 5, 'Perf. MCA', style5)
ws2.write(7, 6, 'CA', style5)
ws2.write(7, 7, 'PCA', style5)
ws2.write(7, 8, 'Perf. PCA', style5)
ws2.write(7, 9, 'SCA', style5)
ws2.write(7, 10, 'AICA', style5)
ws2.write(7, 11, 'PICA', style3)
ws2.write(7, 12, 'ACA', style5_2)
ws2.write(7, 13, 'Perf. ACA', style5_2)
ws2.write(7, 14, 'Ant. MCA', style5_2)
ws2.write(7, 15, 'Post. MCA', style5_2)
ws2.write(7, 16, 'Perf. MCA', style5_2)
ws2.write(7, 17, 'CA', style5_2)
ws2.write(7, 18, 'PCA', style5_2)
ws2.write(7, 19, 'Perf. PCA', style5_2)
ws2.write(7, 20, 'SCA', style5_2)
ws2.write(7, 21, 'AICA', style5_2)
ws2.write(7, 22, 'PICA', style3_2)


i = 0
j = 0
e = 0

while i < numb_anatomic_files:

	if j == 20:

		ws2.write(i+8+e, 0, 'Subject', style9)
		ws2.write(i+8+e, 1, 'ACA', style5_head)
		ws2.write(i+8+e, 2, 'Perf. ACA', style5_head)
		ws2.write(i+8+e, 3, 'Ant. MCA', style5_head)
		ws2.write(i+8+e, 4, 'Post. MCA', style5_head)
		ws2.write(i+8+e, 5, 'Perf. MCA', style5_head)
		ws2.write(i+8+e, 6, 'CA', style5_head)
		ws2.write(i+8+e, 7, 'PCA', style5_head)
		ws2.write(i+8+e, 8, 'Perf. PCA', style5_head)
		ws2.write(i+8+e, 9, 'SCA', style5_head)
		ws2.write(i+8+e, 10, 'AICA', style5_head)
		ws2.write(i+8+e, 11, 'PICA', style3_head)
		ws2.write(i+8+e, 12, 'ACA', style5_2_head)
		ws2.write(i+8+e, 13, 'Perf. ACA', style5_2_head)
		ws2.write(i+8+e, 14, 'Ant. MCA', style5_2_head)
		ws2.write(i+8+e, 15, 'Post. MCA', style5_2_head)
		ws2.write(i+8+e, 16, 'Perf. MCA', style5_2_head)
		ws2.write(i+8+e, 17, 'CA', style5_2_head)
		ws2.write(i+8+e, 18, 'PCA', style5_2_head)
		ws2.write(i+8+e, 19, 'Perf. PCA', style5_2_head)
		ws2.write(i+8+e, 20, 'SCA', style5_2_head)
		ws2.write(i+8+e, 21, 'AICA', style5_2_head)
		ws2.write(i+8+e, 22, 'PICA', style3_2_head)

		j = 0
		e += 1

	else:

		None

	ws2.write(i+8+e, 0, name_files[i], style7)
	ws2.write(i+8+e, 1, lesion_locat_vol[0][i], style8)
	ws2.write(i+8+e, 2, lesion_locat_vol[1][i], style8)
	ws2.write(i+8+e, 3, lesion_locat_vol[4][i], style8)
	ws2.write(i+8+e, 4, lesion_locat_vol[6][i], style8)
	ws2.write(i+8+e, 5, lesion_locat_vol[5][i], style8)
	ws2.write(i+8+e, 6, lesion_locat_vol[3][i], style8)
	ws2.write(i+8+e, 7, lesion_locat_vol[7][i], style8)
	ws2.write(i+8+e, 8, lesion_locat_vol[8][i], style8)
	ws2.write(i+8+e, 9, lesion_locat_vol[10][i], style8)
	ws2.write(i+8+e, 10, lesion_locat_vol[2][i], style8)
	ws2.write(i+8+e, 11, lesion_locat_vol[9][i], style7)
	ws2.write(i+8+e, 12, lesion_locat_vol[11][i], style8)
	ws2.write(i+8+e, 13, lesion_locat_vol[12][i], style8)
	ws2.write(i+8+e, 14, lesion_locat_vol[15][i], style8)
	ws2.write(i+8+e, 15, lesion_locat_vol[17][i], style8)
	ws2.write(i+8+e, 16, lesion_locat_vol[16][i], style8)
	ws2.write(i+8+e, 17, lesion_locat_vol[14][i], style8)
	ws2.write(i+8+e, 18, lesion_locat_vol[18][i], style8)
	ws2.write(i+8+e, 19, lesion_locat_vol[19][i], style8)
	ws2.write(i+8+e, 20, lesion_locat_vol[21][i], style8)
	ws2.write(i+8+e, 21, lesion_locat_vol[13][i], style8)
	ws2.write(i+8+e, 22, lesion_locat_vol[20][i], style7)

	i += 1
	j += 1


ws2.write(i+8+e, 0, '', style10)
ws2.write(i+8+e, 1, '', style10)
ws2.write(i+8+e, 2, '', style10)
ws2.write(i+8+e, 3, '', style10)
ws2.write(i+8+e, 4, '', style10)
ws2.write(i+8+e, 5, '', style10)
ws2.write(i+8+e, 6, '', style10)
ws2.write(i+8+e, 7, '', style10)
ws2.write(i+8+e, 8, '', style10)
ws2.write(i+8+e, 9, '', style10)
ws2.write(i+8+e, 10, '', style10)
ws2.write(i+8+e, 11, '', style10)
ws2.write(i+8+e, 12, '', style10)
ws2.write(i+8+e, 13, '', style10)
ws2.write(i+8+e, 14, '', style10)
ws2.write(i+8+e, 15, '', style10)
ws2.write(i+8+e, 16, '', style10)
ws2.write(i+8+e, 17, '', style10)
ws2.write(i+8+e, 18, '', style10)
ws2.write(i+8+e, 19, '', style10)
ws2.write(i+8+e, 20, '', style10)
ws2.write(i+8+e, 21, '', style10)
ws2.write(i+8+e, 22, '', style10)


# write sheet 3

ws3 = wb.add_sheet("Territory affectation",cell_overwrite_ok=True)

ws3.write_merge(0, 2, 0, 10, 'Territory affectation', style1)
ws3.write_merge(3, 4, 0, 10, 'Percentage of each arterial territory affected by the lesion', style2)
ws3.write_merge(6, 6, 1, 11, 'Left hemisphere', style6)
ws3.write_merge(6, 6, 12, 22, 'Right hemisphere', style4)
ws3.write(7, 0, 'Subject', style9)
ws3.write(7, 1, 'ACA', style5)
ws3.write(7, 2, 'Perf. ACA', style5)
ws3.write(7, 3, 'Ant. MCA', style5)
ws3.write(7, 4, 'Post. MCA', style5)
ws3.write(7, 5, 'Perf. MCA', style5)
ws3.write(7, 6, 'CA', style5)
ws3.write(7, 7, 'PCA', style5)
ws3.write(7, 8, 'Perf. PCA', style5)
ws3.write(7, 9, 'SCA', style5)
ws3.write(7, 10, 'AICA', style5)
ws3.write(7, 11, 'PICA', style3)
ws3.write(7, 12, 'ACA', style5_2)
ws3.write(7, 13, 'Perf. ACA', style5_2)
ws3.write(7, 14, 'Ant. MCA', style5_2)
ws3.write(7, 15, 'Post. MCA', style5_2)
ws3.write(7, 16, 'Perf. MCA', style5_2)
ws3.write(7, 17, 'CA', style5_2)
ws3.write(7, 18, 'PCA', style5_2)
ws3.write(7, 19, 'Perf. PCA', style5_2)
ws3.write(7, 20, 'SCA', style5_2)
ws3.write(7, 21, 'AICA', style5_2)
ws3.write(7, 22, 'PICA', style3_2)


i = 0
j = 0
e = 0

while i < numb_anatomic_files:

	if j == 20:

		ws3.write(i+8+e, 0, 'Subject', style9)
		ws3.write(i+8+e, 1, 'ACA', style5_head)
		ws3.write(i+8+e, 2, 'Perf. ACA', style5_head)
		ws3.write(i+8+e, 3, 'Ant. MCA', style5_head)
		ws3.write(i+8+e, 4, 'Post. MCA', style5_head)
		ws3.write(i+8+e, 5, 'Perf. MCA', style5_head)
		ws3.write(i+8+e, 6, 'CA', style5_head)
		ws3.write(i+8+e, 7, 'PCA', style5_head)
		ws3.write(i+8+e, 8, 'Perf. PCA', style5_head)
		ws3.write(i+8+e, 9, 'SCA', style5_head)
		ws3.write(i+8+e, 10, 'AICA', style5_head)
		ws3.write(i+8+e, 11, 'PICA', style3_head)
		ws3.write(i+8+e, 12, 'ACA', style5_2_head)
		ws3.write(i+8+e, 13, 'Perf. ACA', style5_2_head)
		ws3.write(i+8+e, 14, 'Ant. MCA', style5_2_head)
		ws3.write(i+8+e, 15, 'Post. MCA', style5_2_head)
		ws3.write(i+8+e, 16, 'Perf. MCA', style5_2_head)
		ws3.write(i+8+e, 17, 'CA', style5_2_head)
		ws3.write(i+8+e, 18, 'PCA', style5_2_head)
		ws3.write(i+8+e, 19, 'Perf. PCA', style5_2_head)
		ws3.write(i+8+e, 20, 'SCA', style5_2_head)
		ws3.write(i+8+e, 21, 'AICA', style5_2_head)
		ws3.write(i+8+e, 22, 'PICA', style3_2_head)

		j = 0
		e += 1

	else:

		None
	
	ws3.write(i+8+e, 0, name_files[i], style7)
	ws3.write(i+8+e, 1, lesion_terr_rel_vol[0][i], style8)
	ws3.write(i+8+e, 2, lesion_terr_rel_vol[1][i], style8)
	ws3.write(i+8+e, 3, lesion_terr_rel_vol[4][i], style8)
	ws3.write(i+8+e, 4, lesion_terr_rel_vol[6][i], style8)
	ws3.write(i+8+e, 5, lesion_terr_rel_vol[5][i], style8)
	ws3.write(i+8+e, 6, lesion_terr_rel_vol[3][i], style8)
	ws3.write(i+8+e, 7, lesion_terr_rel_vol[7][i], style8)
	ws3.write(i+8+e, 8, lesion_terr_rel_vol[8][i], style8)
	ws3.write(i+8+e, 9, lesion_terr_rel_vol[10][i], style8)
	ws3.write(i+8+e, 10, lesion_terr_rel_vol[2][i], style8)
	ws3.write(i+8+e, 11, lesion_terr_rel_vol[9][i], style7)
	ws3.write(i+8+e, 12, lesion_terr_rel_vol[11][i], style8)
	ws3.write(i+8+e, 13, lesion_terr_rel_vol[12][i], style8)
	ws3.write(i+8+e, 14, lesion_terr_rel_vol[15][i], style8)
	ws3.write(i+8+e, 15, lesion_terr_rel_vol[17][i], style8)
	ws3.write(i+8+e, 16, lesion_terr_rel_vol[16][i], style8)
	ws3.write(i+8+e, 17, lesion_terr_rel_vol[14][i], style8)
	ws3.write(i+8+e, 18, lesion_terr_rel_vol[18][i], style8)
	ws3.write(i+8+e, 19, lesion_terr_rel_vol[19][i], style8)
	ws3.write(i+8+e, 20, lesion_terr_rel_vol[21][i], style8)
	ws3.write(i+8+e, 21, lesion_terr_rel_vol[13][i], style8)
	ws3.write(i+8+e, 22, lesion_terr_rel_vol[20][i], style7)

	i += 1
	j += 1


ws3.write(i+8+e, 0, '', style10)
ws3.write(i+8+e, 1, '', style10)
ws3.write(i+8+e, 2, '', style10)
ws3.write(i+8+e, 3, '', style10)
ws3.write(i+8+e, 4, '', style10)
ws3.write(i+8+e, 5, '', style10)
ws3.write(i+8+e, 6, '', style10)
ws3.write(i+8+e, 7, '', style10)
ws3.write(i+8+e, 8, '', style10)
ws3.write(i+8+e, 9, '', style10)
ws3.write(i+8+e, 10, '', style10)
ws3.write(i+8+e, 11, '', style10)
ws3.write(i+8+e, 12, '', style10)
ws3.write(i+8+e, 13, '', style10)
ws3.write(i+8+e, 14, '', style10)
ws3.write(i+8+e, 15, '', style10)
ws3.write(i+8+e, 16, '', style10)
ws3.write(i+8+e, 17, '', style10)
ws3.write(i+8+e, 18, '', style10)
ws3.write(i+8+e, 19, '', style10)
ws3.write(i+8+e, 20, '', style10)
ws3.write(i+8+e, 21, '', style10)
ws3.write(i+8+e, 22, '', style10)


# save spreadsheet

wb.save(files_dir + '/results/' + savename + '.xls')

print('Done\n')